# coding=utf-8
from collections.abc import MutableMapping
from io import IOBase
import json
from typing import Any, Callable, IO, Optional, TypeVar, Union, overload

from corehttp.exceptions import (
    ClientAuthenticationError,
    HttpResponseError,
    ResourceExistsError,
    ResourceNotFoundError,
    ResourceNotModifiedError,
    StreamClosedError,
    StreamConsumedError,
    map_error,
)
from corehttp.rest import HttpRequest, HttpResponse
from corehttp.runtime import PipelineClient
from corehttp.runtime.pipeline import PipelineResponse
from corehttp.utils import case_insensitive_dict

from ... import models as _models2
from ..._configuration import BytesClientConfiguration
from ..._utils.model_base import SdkJSONEncoder, _deserialize
from ..._utils.serialization import Deserializer, Serializer

JSON = MutableMapping[str, Any]
T = TypeVar("T")
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, HttpResponse], T, dict[str, Any]], Any]]

_SERIALIZER = Serializer()
_SERIALIZER.client_side_validation = False


def build_property_default_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/encode/bytes/property/default"

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_property_base64_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/encode/bytes/property/base64"

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_property_base64_url_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/encode/bytes/property/base64url"

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


def build_property_base64_url_array_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/encode/bytes/property/base64url-array"

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, headers=_headers, **kwargs)


class PropertyOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~encode.bytes.BytesClient`'s
        :attr:`property` attribute.
    """

    def __init__(self, *args, **kwargs) -> None:
        input_args = list(args)
        self._client: PipelineClient = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config: BytesClientConfiguration = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize: Serializer = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize: Deserializer = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @overload
    def default(
        self, body: _models2.DefaultBytesProperty, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models2.DefaultBytesProperty:
        """default.

        :param body: Required.
        :type body: ~encode.bytes.models.DefaultBytesProperty
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: DefaultBytesProperty. The DefaultBytesProperty is compatible with MutableMapping
        :rtype: ~encode.bytes.models.DefaultBytesProperty
        :raises ~corehttp.exceptions.HttpResponseError:
        """

    @overload
    def default(
        self, body: JSON, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models2.DefaultBytesProperty:
        """default.

        :param body: Required.
        :type body: JSON
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: DefaultBytesProperty. The DefaultBytesProperty is compatible with MutableMapping
        :rtype: ~encode.bytes.models.DefaultBytesProperty
        :raises ~corehttp.exceptions.HttpResponseError:
        """

    @overload
    def default(
        self, body: IO[bytes], *, content_type: str = "application/json", **kwargs: Any
    ) -> _models2.DefaultBytesProperty:
        """default.

        :param body: Required.
        :type body: IO[bytes]
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: DefaultBytesProperty. The DefaultBytesProperty is compatible with MutableMapping
        :rtype: ~encode.bytes.models.DefaultBytesProperty
        :raises ~corehttp.exceptions.HttpResponseError:
        """

    def default(
        self, body: Union[_models2.DefaultBytesProperty, JSON, IO[bytes]], **kwargs: Any
    ) -> _models2.DefaultBytesProperty:
        """default.

        :param body: Is one of the following types: DefaultBytesProperty, JSON, IO[bytes] Required.
        :type body: ~encode.bytes.models.DefaultBytesProperty or JSON or IO[bytes]
        :return: DefaultBytesProperty. The DefaultBytesProperty is compatible with MutableMapping
        :rtype: ~encode.bytes.models.DefaultBytesProperty
        :raises ~corehttp.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models2.DefaultBytesProperty] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _content = None
        if isinstance(body, (IOBase, bytes)):
            _content = body
        else:
            _content = json.dumps(body, cls=SdkJSONEncoder, exclude_readonly=True)  # type: ignore

        _request = build_property_default_request(
            content_type=content_type,
            content=_content,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _decompress = kwargs.pop("decompress", True)
        _stream = kwargs.pop("stream", False)
        pipeline_response: PipelineResponse = self._client.pipeline.run(_request, stream=_stream, **kwargs)

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            if _stream:
                try:
                    response.read()  # Load the body in memory and close the socket
                except (StreamConsumedError, StreamClosedError):
                    pass
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if _stream:
            deserialized = response.iter_bytes() if _decompress else response.iter_raw()
        else:
            deserialized = _deserialize(_models2.DefaultBytesProperty, response.json())

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @overload
    def base64(
        self, body: _models2.Base64BytesProperty, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models2.Base64BytesProperty:
        """base64.

        :param body: Required.
        :type body: ~encode.bytes.models.Base64BytesProperty
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: Base64BytesProperty. The Base64BytesProperty is compatible with MutableMapping
        :rtype: ~encode.bytes.models.Base64BytesProperty
        :raises ~corehttp.exceptions.HttpResponseError:
        """

    @overload
    def base64(
        self, body: JSON, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models2.Base64BytesProperty:
        """base64.

        :param body: Required.
        :type body: JSON
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: Base64BytesProperty. The Base64BytesProperty is compatible with MutableMapping
        :rtype: ~encode.bytes.models.Base64BytesProperty
        :raises ~corehttp.exceptions.HttpResponseError:
        """

    @overload
    def base64(
        self, body: IO[bytes], *, content_type: str = "application/json", **kwargs: Any
    ) -> _models2.Base64BytesProperty:
        """base64.

        :param body: Required.
        :type body: IO[bytes]
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: Base64BytesProperty. The Base64BytesProperty is compatible with MutableMapping
        :rtype: ~encode.bytes.models.Base64BytesProperty
        :raises ~corehttp.exceptions.HttpResponseError:
        """

    def base64(
        self, body: Union[_models2.Base64BytesProperty, JSON, IO[bytes]], **kwargs: Any
    ) -> _models2.Base64BytesProperty:
        """base64.

        :param body: Is one of the following types: Base64BytesProperty, JSON, IO[bytes] Required.
        :type body: ~encode.bytes.models.Base64BytesProperty or JSON or IO[bytes]
        :return: Base64BytesProperty. The Base64BytesProperty is compatible with MutableMapping
        :rtype: ~encode.bytes.models.Base64BytesProperty
        :raises ~corehttp.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models2.Base64BytesProperty] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _content = None
        if isinstance(body, (IOBase, bytes)):
            _content = body
        else:
            _content = json.dumps(body, cls=SdkJSONEncoder, exclude_readonly=True)  # type: ignore

        _request = build_property_base64_request(
            content_type=content_type,
            content=_content,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _decompress = kwargs.pop("decompress", True)
        _stream = kwargs.pop("stream", False)
        pipeline_response: PipelineResponse = self._client.pipeline.run(_request, stream=_stream, **kwargs)

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            if _stream:
                try:
                    response.read()  # Load the body in memory and close the socket
                except (StreamConsumedError, StreamClosedError):
                    pass
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if _stream:
            deserialized = response.iter_bytes() if _decompress else response.iter_raw()
        else:
            deserialized = _deserialize(_models2.Base64BytesProperty, response.json())

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @overload
    def base64_url(
        self, body: _models2.Base64urlBytesProperty, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models2.Base64urlBytesProperty:
        """base64_url.

        :param body: Required.
        :type body: ~encode.bytes.models.Base64urlBytesProperty
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: Base64urlBytesProperty. The Base64urlBytesProperty is compatible with MutableMapping
        :rtype: ~encode.bytes.models.Base64urlBytesProperty
        :raises ~corehttp.exceptions.HttpResponseError:
        """

    @overload
    def base64_url(
        self, body: JSON, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models2.Base64urlBytesProperty:
        """base64_url.

        :param body: Required.
        :type body: JSON
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: Base64urlBytesProperty. The Base64urlBytesProperty is compatible with MutableMapping
        :rtype: ~encode.bytes.models.Base64urlBytesProperty
        :raises ~corehttp.exceptions.HttpResponseError:
        """

    @overload
    def base64_url(
        self, body: IO[bytes], *, content_type: str = "application/json", **kwargs: Any
    ) -> _models2.Base64urlBytesProperty:
        """base64_url.

        :param body: Required.
        :type body: IO[bytes]
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: Base64urlBytesProperty. The Base64urlBytesProperty is compatible with MutableMapping
        :rtype: ~encode.bytes.models.Base64urlBytesProperty
        :raises ~corehttp.exceptions.HttpResponseError:
        """

    def base64_url(
        self, body: Union[_models2.Base64urlBytesProperty, JSON, IO[bytes]], **kwargs: Any
    ) -> _models2.Base64urlBytesProperty:
        """base64_url.

        :param body: Is one of the following types: Base64urlBytesProperty, JSON, IO[bytes] Required.
        :type body: ~encode.bytes.models.Base64urlBytesProperty or JSON or IO[bytes]
        :return: Base64urlBytesProperty. The Base64urlBytesProperty is compatible with MutableMapping
        :rtype: ~encode.bytes.models.Base64urlBytesProperty
        :raises ~corehttp.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models2.Base64urlBytesProperty] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _content = None
        if isinstance(body, (IOBase, bytes)):
            _content = body
        else:
            _content = json.dumps(body, cls=SdkJSONEncoder, exclude_readonly=True)  # type: ignore

        _request = build_property_base64_url_request(
            content_type=content_type,
            content=_content,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _decompress = kwargs.pop("decompress", True)
        _stream = kwargs.pop("stream", False)
        pipeline_response: PipelineResponse = self._client.pipeline.run(_request, stream=_stream, **kwargs)

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            if _stream:
                try:
                    response.read()  # Load the body in memory and close the socket
                except (StreamConsumedError, StreamClosedError):
                    pass
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if _stream:
            deserialized = response.iter_bytes() if _decompress else response.iter_raw()
        else:
            deserialized = _deserialize(_models2.Base64urlBytesProperty, response.json())

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @overload
    def base64_url_array(
        self, body: _models2.Base64urlArrayBytesProperty, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models2.Base64urlArrayBytesProperty:
        """base64_url_array.

        :param body: Required.
        :type body: ~encode.bytes.models.Base64urlArrayBytesProperty
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: Base64urlArrayBytesProperty. The Base64urlArrayBytesProperty is compatible with
         MutableMapping
        :rtype: ~encode.bytes.models.Base64urlArrayBytesProperty
        :raises ~corehttp.exceptions.HttpResponseError:
        """

    @overload
    def base64_url_array(
        self, body: JSON, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models2.Base64urlArrayBytesProperty:
        """base64_url_array.

        :param body: Required.
        :type body: JSON
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: Base64urlArrayBytesProperty. The Base64urlArrayBytesProperty is compatible with
         MutableMapping
        :rtype: ~encode.bytes.models.Base64urlArrayBytesProperty
        :raises ~corehttp.exceptions.HttpResponseError:
        """

    @overload
    def base64_url_array(
        self, body: IO[bytes], *, content_type: str = "application/json", **kwargs: Any
    ) -> _models2.Base64urlArrayBytesProperty:
        """base64_url_array.

        :param body: Required.
        :type body: IO[bytes]
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: Base64urlArrayBytesProperty. The Base64urlArrayBytesProperty is compatible with
         MutableMapping
        :rtype: ~encode.bytes.models.Base64urlArrayBytesProperty
        :raises ~corehttp.exceptions.HttpResponseError:
        """

    def base64_url_array(
        self, body: Union[_models2.Base64urlArrayBytesProperty, JSON, IO[bytes]], **kwargs: Any
    ) -> _models2.Base64urlArrayBytesProperty:
        """base64_url_array.

        :param body: Is one of the following types: Base64urlArrayBytesProperty, JSON, IO[bytes]
         Required.
        :type body: ~encode.bytes.models.Base64urlArrayBytesProperty or JSON or IO[bytes]
        :return: Base64urlArrayBytesProperty. The Base64urlArrayBytesProperty is compatible with
         MutableMapping
        :rtype: ~encode.bytes.models.Base64urlArrayBytesProperty
        :raises ~corehttp.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models2.Base64urlArrayBytesProperty] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _content = None
        if isinstance(body, (IOBase, bytes)):
            _content = body
        else:
            _content = json.dumps(body, cls=SdkJSONEncoder, exclude_readonly=True)  # type: ignore

        _request = build_property_base64_url_array_request(
            content_type=content_type,
            content=_content,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _decompress = kwargs.pop("decompress", True)
        _stream = kwargs.pop("stream", False)
        pipeline_response: PipelineResponse = self._client.pipeline.run(_request, stream=_stream, **kwargs)

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            if _stream:
                try:
                    response.read()  # Load the body in memory and close the socket
                except (StreamConsumedError, StreamClosedError):
                    pass
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if _stream:
            deserialized = response.iter_bytes() if _decompress else response.iter_raw()
        else:
            deserialized = _deserialize(_models2.Base64urlArrayBytesProperty, response.json())

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
# pylint: disable=useless-super-delegation

import datetime
from typing import Any, Mapping, overload

from .._utils.model_base import Model as _Model, rest_field


class BlobProperties(_Model):
    """Properties of a blob.

    :ivar name: Required.
    :vartype name: str
    :ivar size: Required.
    :vartype size: int
    :ivar content_type: Required.
    :vartype content_type: str
    :ivar created_on: Required.
    :vartype created_on: ~datetime.datetime
    """

    name: str = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Required."""
    size: int = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Required."""
    content_type: str = rest_field(name="contentType", visibility=["read", "create", "update", "delete", "query"])
    """Required."""
    created_on: datetime.datetime = rest_field(
        name="createdOn", visibility=["read", "create", "update", "delete", "query"], format="rfc3339"
    )
    """Required."""

    @overload
    def __init__(
        self,
        *,
        name: str,
        size: int,
        content_type: str,
        created_on: datetime.datetime,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class Input(_Model):
    """Input.

    :ivar name: Required.
    :vartype name: str
    """

    name: str = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Required."""

    @overload
    def __init__(
        self,
        *,
        name: str,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class WithBodyRequest(_Model):
    """WithBodyRequest.

    :ivar name: Required.
    :vartype name: str
    """

    name: str = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Required."""

    @overload
    def __init__(
        self,
        *,
        name: str,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)



# Copyright (c) Microsoft Corporation.
# Licensed under the MIT license.
"""Base builder infrastructure: lifecycle state, base class, and generic builder."""

from __future__ import annotations

from copy import deepcopy
from enum import Enum
from typing import TYPE_CHECKING, Any, cast

from ...models import _generated as generated_models

if TYPE_CHECKING:
    from .._event_stream import ResponseEventStream


def _require_non_empty(value: str, field_name: str) -> str:
    """Validate that a string value is non-empty.

    :param value: The string value to check.
    :type value: str
    :param field_name: The field name to include in the error message.
    :type field_name: str
    :returns: The validated non-empty string.
    :rtype: str
    :raises ValueError: If *value* is not a non-empty string.
    """
    if not isinstance(value, str) or not value.strip():
        raise ValueError(f"{field_name} must be a non-empty string")
    return value


class BuilderLifecycleState(Enum):
    NOT_STARTED = "not_started"
    ADDED = "added"
    DONE = "done"


class BaseOutputItemBuilder:
    """Base output-item builder with lifecycle guards for added/done events."""

    def __init__(self, stream: "ResponseEventStream", output_index: int, item_id: str) -> None:
        """Initialize the base output-item builder.

        :param stream: The parent event stream to emit events into.
        :type stream: ResponseEventStream
        :param output_index: The zero-based index of this output item.
        :type output_index: int
        :param item_id: Unique identifier for this output item.
        :type item_id: str
        """
        self._stream = stream
        self._output_index = output_index
        self._item_id = item_id
        self._lifecycle_state = BuilderLifecycleState.NOT_STARTED

    @property
    def item_id(self) -> str:
        """Return the output item identifier.

        :returns: The item ID.
        :rtype: str
        """
        return self._item_id

    @property
    def output_index(self) -> int:
        """Return the zero-based output index.

        :returns: The output index.
        :rtype: int
        """
        return self._output_index

    def _ensure_transition(self, expected: BuilderLifecycleState, new_state: BuilderLifecycleState) -> None:
        """Guard a lifecycle state transition.

        :param expected: The expected current lifecycle state.
        :type expected: BuilderLifecycleState
        :param new_state: The target state to transition to.
        :type new_state: BuilderLifecycleState
        :rtype: None
        :raises ValueError: If the current state does not match *expected*.
        """
        if self._lifecycle_state is not expected:
            raise ValueError(
                "cannot transition to "
                f"'{new_state.value}' from '{self._lifecycle_state.value}' "
                f"(expected '{expected.value}')"
            )
        self._lifecycle_state = new_state

    def _emit_added(self, item: dict[str, Any]) -> generated_models.ResponseOutputItemAddedEvent:
        """Emit an ``output_item.added`` event with lifecycle guard.

        :param item: The output item dict to include in the event.
        :type item: dict[str, Any]
        :returns: The emitted event.
        :rtype: ResponseOutputItemAddedEvent
        :raises ValueError: If the builder is not in ``NOT_STARTED`` state.
        """
        self._ensure_transition(BuilderLifecycleState.NOT_STARTED, BuilderLifecycleState.ADDED)
        stamped_item = self._stream._with_output_item_defaults(item)  # pylint: disable=protected-access
        return cast(
            generated_models.ResponseOutputItemAddedEvent,
            self._stream._emit_event(  # pylint: disable=protected-access
                {
                    "type": generated_models.ResponseStreamEventType.RESPONSE_OUTPUT_ITEM_ADDED.value,
                    "output_index": self._output_index,
                    "item": stamped_item,
                }
            ),
        )

    def _emit_done(self, item: dict[str, Any]) -> generated_models.ResponseOutputItemDoneEvent:
        """Emit an ``output_item.done`` event with lifecycle guard.

        :param item: The completed output item dict to include in the event.
        :type item: dict[str, Any]
        :returns: The emitted event.
        :rtype: ResponseOutputItemDoneEvent
        :raises ValueError: If the builder is not in ``ADDED`` state.
        """
        self._ensure_transition(BuilderLifecycleState.ADDED, BuilderLifecycleState.DONE)
        stamped_item = self._stream._with_output_item_defaults(item)  # pylint: disable=protected-access
        return cast(
            generated_models.ResponseOutputItemDoneEvent,
            self._stream._emit_event(  # pylint: disable=protected-access
                {
                    "type": generated_models.ResponseStreamEventType.RESPONSE_OUTPUT_ITEM_DONE.value,
                    "output_index": self._output_index,
                    "item": stamped_item,
                }
            ),
        )

    def _emit_item_state_event(
        self, event_type: str, *, extra_payload: dict[str, Any] | None = None
    ) -> generated_models.ResponseStreamEvent:
        """Emit an item-level state event (e.g., in-progress, searching, completed).

        :param event_type: The event type string.
        :type event_type: str
        :keyword extra_payload: Optional additional fields to merge.
        :paramtype extra_payload: dict[str, Any] | None
        :returns: The emitted event.
        :rtype: ResponseStreamEvent
        """
        event: dict[str, Any] = {
            "type": event_type,
            "item_id": self._item_id,
            "output_index": self._output_index,
        }
        if extra_payload:
            event.update(deepcopy(extra_payload))
        return self._stream._emit_event(event)  # pylint: disable=protected-access


class OutputItemBuilder(BaseOutputItemBuilder):
    """Generic output-item builder for item types without dedicated scoped builders."""

    def emit_added(self, item: generated_models.OutputItem) -> generated_models.ResponseOutputItemAddedEvent:
        """Emit an ``output_item.added`` event for a generic item.

        :param item: The output item model instance.
        :type item: OutputItem
        :returns: The emitted event.
        :rtype: ResponseOutputItemAddedEvent
        """
        return self._emit_added(item.as_dict())

    def emit_done(self, item: generated_models.OutputItem) -> generated_models.ResponseOutputItemDoneEvent:
        """Emit an ``output_item.done`` event for a generic item.

        :param item: The completed output item model instance.
        :type item: OutputItem
        :returns: The emitted event.
        :rtype: ResponseOutputItemDoneEvent
        """
        return self._emit_done(item.as_dict())



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from devtools_testutils.aio import recorded_by_proxy_async
from testpreparer import SpecialWordsPreparer
from testpreparer_async import SpecialWordsClientTestBaseAsync


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestSpecialWordsModelPropertiesOperationsAsync(SpecialWordsClientTestBaseAsync):
    @SpecialWordsPreparer()
    @recorded_by_proxy_async
    async def test_model_properties_same_as_model(self, specialwords_endpoint):
        client = self.create_async_client(endpoint=specialwords_endpoint)
        response = await client.model_properties.same_as_model(
            body={"SameAsModel": "str"},
        )

        # please add some check logic here by yourself
        # ...

    @SpecialWordsPreparer()
    @recorded_by_proxy_async
    async def test_model_properties_dict_methods(self, specialwords_endpoint):
        client = self.create_async_client(endpoint=specialwords_endpoint)
        response = await client.model_properties.dict_methods(
            body={
                "clear": "str",
                "copy": "str",
                "get": "str",
                "items": "str",
                "keys": "str",
                "pop": "str",
                "popitem": "str",
                "setdefault": "str",
                "update": "str",
                "values": "str",
            },
        )

        # please add some check logic here by yourself
        # ...

    @SpecialWordsPreparer()
    @recorded_by_proxy_async
    async def test_model_properties_with_list(self, specialwords_endpoint):
        client = self.create_async_client(endpoint=specialwords_endpoint)
        response = await client.model_properties.with_list(
            body={"list": "str"},
        )

        # please add some check logic here by yourself
        # ...



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.azurestackhcivm import AzureStackHCIVmClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer, recorded_by_proxy

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestAzureStackHCIVmVirtualHardDisksOperations(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(AzureStackHCIVmClient)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_virtual_hard_disks_get(self, resource_group):
        response = self.client.virtual_hard_disks.get(
            resource_group_name=resource_group.name,
            virtual_hard_disk_name="str",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_virtual_hard_disks_begin_create_or_update(self, resource_group):
        response = self.client.virtual_hard_disks.begin_create_or_update(
            resource_group_name=resource_group.name,
            virtual_hard_disk_name="str",
            resource={
                "location": "str",
                "extendedLocation": {"name": "str", "type": "str"},
                "id": "str",
                "name": "str",
                "properties": {
                    "blockSizeBytes": 0,
                    "containerId": "str",
                    "createFromLocal": bool,
                    "diskFileFormat": "str",
                    "diskSizeGB": 0,
                    "downloadUrl": "str",
                    "dynamic": bool,
                    "hyperVGeneration": "str",
                    "logicalSectorBytes": 0,
                    "maxShares": 0,
                    "physicalSectorBytes": 0,
                    "provisioningState": "str",
                    "status": {
                        "downloadStatus": {"downloadedSizeInMB": 0, "progressPercentage": 0, "status": "str"},
                        "errorCode": "str",
                        "errorMessage": "str",
                        "managedBy": ["str"],
                        "provisioningStatus": {"operationId": "str", "status": "str"},
                        "uniqueId": "str",
                        "uploadStatus": {
                            "errorCode": "str",
                            "errorMessage": "str",
                            "progressPercentage": 0,
                            "status": "str",
                            "uploadedSizeInMB": 0,
                        },
                    },
                },
                "systemData": {
                    "createdAt": "2020-02-20 00:00:00",
                    "createdBy": "str",
                    "createdByType": "str",
                    "lastModifiedAt": "2020-02-20 00:00:00",
                    "lastModifiedBy": "str",
                    "lastModifiedByType": "str",
                },
                "tags": {"str": "str"},
                "type": "str",
            },
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_virtual_hard_disks_begin_update(self, resource_group):
        response = self.client.virtual_hard_disks.begin_update(
            resource_group_name=resource_group.name,
            virtual_hard_disk_name="str",
            properties={"properties": {"diskSizeGB": 0}, "tags": {"str": "str"}},
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_virtual_hard_disks_begin_delete(self, resource_group):
        response = self.client.virtual_hard_disks.begin_delete(
            resource_group_name=resource_group.name,
            virtual_hard_disk_name="str",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_virtual_hard_disks_list_by_resource_group(self, resource_group):
        response = self.client.virtual_hard_disks.list_by_resource_group(
            resource_group_name=resource_group.name,
        )
        result = [r for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_virtual_hard_disks_list_all(self, resource_group):
        response = self.client.virtual_hard_disks.list_all()
        result = [r for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_virtual_hard_disks_begin_upload(self, resource_group):
        response = self.client.virtual_hard_disks.begin_upload(
            resource_group_name=resource_group.name,
            virtual_hard_disk_name="str",
            body={"azureManagedDiskUploadUrl": "str"},
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...



# ------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# -------------------------------------------------------------------------
import datetime
from typing import (
    Any,
    Awaitable,
    Callable,
    Mapping,
    Optional,
    overload,
    List,
    Tuple,
)
from azure.core.credentials_async import AsyncTokenCredential
from .._constants import (
    DEFAULT_STARTUP_TIMEOUT,
)
from .._models import AzureAppConfigurationKeyVaultOptions, SettingSelector
from .._utils import (
    delay_failure,
    process_load_parameters,
    sdk_allowed_kwargs,
)
from ._azureappconfigurationproviderasync import (
    _buildprovider,
    AzureAppConfigurationProvider,
)

JSON = Mapping[str, Any]


@overload
async def load(  # pylint: disable=docstring-keyword-should-match-keyword-only
    endpoint: str,
    credential: "AsyncTokenCredential",
    *,
    selects: Optional[List[SettingSelector]] = None,
    trim_prefixes: Optional[List[str]] = None,
    keyvault_credential: Optional["AsyncTokenCredential"] = None,
    keyvault_client_configs: Optional[Mapping[str, JSON]] = None,
    secret_resolver: Optional[Callable[[str], str]] = None,
    key_vault_options: Optional[AzureAppConfigurationKeyVaultOptions] = None,
    refresh_on: Optional[List[Tuple[str, str]]] = None,
    refresh_interval: int = 30,
    refresh_enabled: Optional[bool] = None,
    on_refresh_success: Optional[Callable] = None,
    on_refresh_error: Optional[Callable[[Exception], Awaitable[None]]] = None,
    feature_flag_enabled: bool = False,
    feature_flag_selectors: Optional[List[SettingSelector]] = None,
    feature_flag_refresh_enabled: bool = False,
    startup_timeout: int = DEFAULT_STARTUP_TIMEOUT,
    **kwargs,
) -> "AzureAppConfigurationProvider":
    """
    Loads configuration settings from Azure App Configuration into a Python application.

    :param str endpoint: Endpoint for App Configuration resource.
    :param ~azure.core.credentials_async.AsyncTokenCredential credential: Credential for App Configuration resource.
    :keyword Optional[List[~azure.appconfiguration.provider.SettingSelector]] selects: List of setting selectors to
    filter configuration settings
    :keyword Optional[List[str]] trim_prefixes: List of prefixes to trim from configuration keys
    :keyword ~azure.core.credentials_async.AsyncTokenCredential keyvault_credential: A credential for authenticating
    with the key vault. This is optional if keyvault_client_configs is provided.
    :keyword Mapping[str, Mapping] keyvault_client_configs: A Mapping of SecretClient endpoints to client
    configurations from azure-keyvault-secrets. This is optional if keyvault_credential is provided. If a credential
    isn't provided a credential will need to be in each set for each.
    :keyword Callable[[str], str] secret_resolver: A function that takes a URI and returns a value.
    :keyword List[Tuple[str, str]] refresh_on: One or more settings whose modification will trigger a full refresh
    after a fixed interval. This should be a list of Key-Label pairs for specific settings (filters and wildcards are
    not supported).
    :keyword int refresh_interval: The minimum time in seconds between when a call to `refresh` will actually trigger a
     service call to update the settings. Default value is 30 seconds.
    :keyword refresh_enabled: Optional flag to enable or disable refreshing of configuration settings. Defaults to
    True if ``refresh_on`` is set, otherwise False.
    :paramtype refresh_enabled: Optional[bool]
    :keyword on_refresh_success: Optional callback to be invoked when a change is found and a successful refresh has
    happened.
    :paramtype on_refresh_success: Optional[Callable]
    :keyword on_refresh_error: Optional callback to be invoked when an error occurs while refreshing settings. If not
    specified, errors will be raised.
    :paramtype on_refresh_error: Optional[Callable[[Exception], Awaitable[None]]]
    :keyword feature_flag_enabled: Optional flag to enable or disable the loading of feature flags. Default is False.
    :paramtype feature_flag_enabled: bool
    :keyword feature_flag_selectors: Optional list of selectors to filter feature flags. By default will load all
     feature flags without a label.
    :paramtype feature_flag_selectors: List[SettingSelector]
    :keyword feature_flag_refresh_enabled: Optional flag to enable or disable the refresh of feature flags. Default is
     False.
    :paramtype feature_flag_refresh_enabled: bool
    :keyword replica_discovery_enabled: Optional flag to enable or disable the discovery of replica endpoints. Default
     is True.
    :paramtype replica_discovery_enabled: bool
    :keyword load_balancing_enabled: Optional flag to enable or disable the load balancing of replica endpoints. Default
     is False.
    :paramtype load_balancing_enabled: bool
    :keyword configuration_mapper: Optional function to map configuration settings. Enables transformation of
    configurations before they are added to the provider.
    :paramtype configuration_mapper: Optional[Callable[[ConfigurationSetting], Awaitable[None]]]
    :keyword startup_timeout: The amount of time in seconds allowed to load data from Azure App Configuration on
     startup. The default value is 100 seconds.
    :paramtype startup_timeout: int
    """


@overload
async def load(  # pylint: disable=docstring-keyword-should-match-keyword-only
    *,
    connection_string: str,
    selects: Optional[List[SettingSelector]] = None,
    trim_prefixes: Optional[List[str]] = None,
    keyvault_credential: Optional["AsyncTokenCredential"] = None,
    keyvault_client_configs: Optional[Mapping[str, JSON]] = None,
    secret_resolver: Optional[Callable[[str], str]] = None,
    key_vault_options: Optional[AzureAppConfigurationKeyVaultOptions] = None,
    refresh_on: Optional[List[Tuple[str, str]]] = None,
    refresh_interval: int = 30,
    refresh_enabled: Optional[bool] = None,
    on_refresh_success: Optional[Callable] = None,
    on_refresh_error: Optional[Callable[[Exception], Awaitable[None]]] = None,
    feature_flag_enabled: bool = False,
    feature_flag_selectors: Optional[List[SettingSelector]] = None,
    feature_flag_refresh_enabled: bool = False,
    startup_timeout: int = DEFAULT_STARTUP_TIMEOUT,
    **kwargs,
) -> "AzureAppConfigurationProvider":
    """
    Loads configuration settings from Azure App Configuration into a Python application.

    :keyword str connection_string: Connection string for App Configuration resource.
    :keyword Optional[List[~azure.appconfiguration.provider.SettingSelector]] selects: List of setting selectors to
    filter configuration settings
    :keyword trim_prefixes: Optional[List[str]] trim_prefixes: List of prefixes to trim from configuration keys
    :keyword ~azure.core.credentials_async.AsyncTokenCredential keyvault_credential: A credential for authenticating
    with the key vault. This is optional if keyvault_client_configs is provided.
    :keyword Mapping[str, Mapping] keyvault_client_configs: A Mapping of SecretClient endpoints to client
    configurations from azure-keyvault-secrets. This is optional if keyvault_credential is provided. If a credential
    isn't provided a credential will need to be in each set for each.
    :keyword Callable[[str], str] secret_resolver: A function that takes a URI and returns a value.
    :keyword List[Tuple[str, str]] refresh_on: One or more settings whose modification will trigger a full refresh
    after a fixed interval. This should be a list of Key-Label pairs for specific settings (filters and wildcards are
    not supported).
    :keyword refresh_on: One or more settings whose modification will trigger a full refresh after a fixed interval.
    This should be a list of Key-Label pairs for specific settings (filters and wildcards are not supported).
    :paramtype refresh_on: List[Tuple[str, str]]
    :keyword int refresh_interval: The minimum time in seconds between when a call to `refresh` will actually trigger a
     service call to update the settings. Default value is 30 seconds.
    :keyword refresh_enabled: Optional flag to enable or disable refreshing of configuration settings. Defaults to
    True if ``refresh_on`` is set, otherwise False.
    :paramtype refresh_enabled: Optional[bool]
    :keyword on_refresh_success: Optional callback to be invoked when a change is found and a successful refresh has
     happened.
    :paramtype on_refresh_success: Optional[Callable]
    :keyword on_refresh_error: Optional callback to be invoked when an error occurs while refreshing settings. If not
    specified, errors will be raised.
    :paramtype on_refresh_error: Optional[Callable[[Exception], Awaitable[None]]]
    :keyword feature_flag_enabled: Optional flag to enable or disable the loading of feature flags. Default is False.
    :paramtype feature_flag_enabled: bool
    :keyword feature_flag_selectors: Optional list of selectors to filter feature flags. By default will load all
     feature flags without a label.
    :paramtype feature_flag_selectors: List[SettingSelector]
    :keyword feature_flag_refresh_enabled: Optional flag to enable or disable the refresh of feature flags. Default is
     False.
    :paramtype feature_flag_refresh_enabled: bool
    :keyword replica_discovery_enabled: Optional flag to enable or disable the discovery of replica endpoints. Default
     is True.
    :paramtype replica_discovery_enabled: bool
    :keyword load_balancing_enabled: Optional flag to enable or disable the load balancing of replica endpoints. Default
     is False.
    :paramtype load_balancing_enabled: bool
    :keyword configuration_mapper: Optional function to map configuration settings. Enables transformation of
    configurations before they are added to the provider.
    :paramtype configuration_mapper: Optional[Callable[[ConfigurationSetting], Awaitable[None]]]
    :keyword startup_timeout: The amount of time in seconds allowed to load data from Azure App Configuration on
     startup. The default value is 100 seconds.
    :paramtype startup_timeout: int
    """


async def load(*args, **kwargs) -> "AzureAppConfigurationProvider":
    start_time = datetime.datetime.now()

    # Process common load parameters using shared logic
    params = process_load_parameters(*args, **kwargs)

    provider = await _buildprovider(
        params["connection_string"],
        params["endpoint"],
        params["credential"],
        uses_key_vault=params["uses_key_vault"],
        startup_timeout=params["startup_timeout"],
        **params["kwargs"],
    )
    kwargs = sdk_allowed_kwargs(params["kwargs"])

    try:
        await provider._load_all(**kwargs)  # pylint:disable=protected-access
    except Exception:
        delay_failure(start_time)
        raise
    return provider



# pylint: disable=too-many-lines,too-many-statements
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import sys
from typing import Any, Callable, Dict, Optional, Type, TypeVar

from azure.core.exceptions import (
    ClientAuthenticationError,
    HttpResponseError,
    ResourceExistsError,
    ResourceNotFoundError,
    ResourceNotModifiedError,
    map_error,
)
from azure.core.pipeline import PipelineResponse
from azure.core.rest import AsyncHttpResponse, HttpRequest
from azure.core.tracing.decorator_async import distributed_trace_async
from azure.core.utils import case_insensitive_dict
from azure.mgmt.core.exceptions import ARMErrorFormat

from ... import models as _models
from ...operations._software_update_configuration_machine_runs_operations import (
    build_get_by_id_request,
    build_list_request,
)

if sys.version_info >= (3, 9):
    from collections.abc import MutableMapping
else:
    from typing import MutableMapping  # type: ignore  # pylint: disable=ungrouped-imports
T = TypeVar("T")
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, AsyncHttpResponse], T, Dict[str, Any]], Any]]


class SoftwareUpdateConfigurationMachineRunsOperations:  # pylint: disable=name-too-long
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~azure.mgmt.automation.aio.AutomationClient`'s
        :attr:`software_update_configuration_machine_runs` attribute.
    """

    models = _models

    def __init__(self, *args, **kwargs) -> None:
        input_args = list(args)
        self._client = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @distributed_trace_async
    async def get_by_id(
        self,
        resource_group_name: str,
        automation_account_name: str,
        software_update_configuration_machine_run_id: str,
        client_request_id: Optional[str] = None,
        **kwargs: Any
    ) -> _models.SoftwareUpdateConfigurationMachineRun:
        """Get a single software update configuration machine run by Id.

        .. seealso::
           - http://aka.ms/azureautomationsdk/softwareupdateconfigurationoperations

        :param resource_group_name: Name of an Azure Resource group. Required.
        :type resource_group_name: str
        :param automation_account_name: The name of the automation account. Required.
        :type automation_account_name: str
        :param software_update_configuration_machine_run_id: The Id of the software update
         configuration machine run. Required.
        :type software_update_configuration_machine_run_id: str
        :param client_request_id: Identifies this specific client request. Default value is None.
        :type client_request_id: str
        :return: SoftwareUpdateConfigurationMachineRun or the result of cls(response)
        :rtype: ~azure.mgmt.automation.models.SoftwareUpdateConfigurationMachineRun
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping[int, Type[HttpResponseError]] = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2022-08-08"))
        cls: ClsType[_models.SoftwareUpdateConfigurationMachineRun] = kwargs.pop("cls", None)

        _request = build_get_by_id_request(
            resource_group_name=resource_group_name,
            automation_account_name=automation_account_name,
            software_update_configuration_machine_run_id=software_update_configuration_machine_run_id,
            subscription_id=self._config.subscription_id,
            client_request_id=client_request_id,
            api_version=api_version,
            headers=_headers,
            params=_params,
        )
        _request.url = self._client.format_url(_request.url)

        _stream = False
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        deserialized = self._deserialize("SoftwareUpdateConfigurationMachineRun", pipeline_response.http_response)

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @distributed_trace_async
    async def list(
        self,
        resource_group_name: str,
        automation_account_name: str,
        client_request_id: Optional[str] = None,
        filter: Optional[str] = None,
        skip: Optional[str] = None,
        top: Optional[str] = None,
        **kwargs: Any
    ) -> _models.SoftwareUpdateConfigurationMachineRunListResult:
        """Return list of software update configuration machine runs.

        .. seealso::
           - http://aka.ms/azureautomationsdk/softwareupdateconfigurationoperations

        :param resource_group_name: Name of an Azure Resource group. Required.
        :type resource_group_name: str
        :param automation_account_name: The name of the automation account. Required.
        :type automation_account_name: str
        :param client_request_id: Identifies this specific client request. Default value is None.
        :type client_request_id: str
        :param filter: The filter to apply on the operation. You can use the following filters:
         'properties/osType', 'properties/status', 'properties/startTime', and
         'properties/softwareUpdateConfiguration/name'. Default value is None.
        :type filter: str
        :param skip: number of entries you skip before returning results. Default value is None.
        :type skip: str
        :param top: Maximum number of entries returned in the results collection. Default value is
         None.
        :type top: str
        :return: SoftwareUpdateConfigurationMachineRunListResult or the result of cls(response)
        :rtype: ~azure.mgmt.automation.models.SoftwareUpdateConfigurationMachineRunListResult
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping[int, Type[HttpResponseError]] = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2022-08-08"))
        cls: ClsType[_models.SoftwareUpdateConfigurationMachineRunListResult] = kwargs.pop("cls", None)

        _request = build_list_request(
            resource_group_name=resource_group_name,
            automation_account_name=automation_account_name,
            subscription_id=self._config.subscription_id,
            client_request_id=client_request_id,
            filter=filter,
            skip=skip,
            top=top,
            api_version=api_version,
            headers=_headers,
            params=_params,
        )
        _request.url = self._client.format_url(_request.url)

        _stream = False
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        deserialized = self._deserialize(
            "SoftwareUpdateConfigurationMachineRunListResult", pipeline_response.http_response
        )

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from enum import Enum
from azure.core import CaseInsensitiveEnumMeta


class EnumV2(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of EnumV2."""

    ENUM_MEMBER_V2 = "enumMemberV2"
    """ENUM_MEMBER_V2."""


class EnumV3(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of EnumV3."""

    ENUM_MEMBER_V1 = "enumMemberV1"
    """ENUM_MEMBER_V1."""
    ENUM_MEMBER_V2_PREVIEW = "enumMemberV2Preview"
    """ENUM_MEMBER_V2_PREVIEW."""


class Versions(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The version of the API."""

    V1 = "v1"
    """The version v1."""
    V2_PREVIEW = "v2preview"
    """The V2 Preview version."""
    V2 = "v2"
    """The version v2."""



# pylint: disable=line-too-long,useless-suppression
# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------
"""
DESCRIPTION:
    This sample demonstrates how to add files to agent during the vector store creation.

USAGE:
    python sample_agents_vector_store_enterprise_file_search_async.py

    Before running the sample:

    pip install azure-ai-projects azure-ai-agents azure-identity azure-ai-ml aiohttp

    Set these environment variables with your own values:
    1) PROJECT_ENDPOINT - The Azure AI Project endpoint, as found in the Overview
                          page of your Azure AI Foundry portal.
    2) MODEL_DEPLOYMENT_NAME - The deployment name of the AI model.
"""
import asyncio
import os

from azure.ai.projects.aio import AIProjectClient
from azure.ai.agents.models import (
    FileSearchTool,
    VectorStoreDataSource,
    VectorStoreDataSourceAssetType,
    ListSortOrder,
)
from azure.identity.aio import DefaultAzureCredential


async def main():
    project_client = AIProjectClient(
        endpoint=os.environ["PROJECT_ENDPOINT"],
        credential=DefaultAzureCredential(),
    )

    async with project_client:
        agents_client = project_client.agents

        # We will upload the local file to Azure and will use it for vector store creation.
        asset_uri = os.environ["AZURE_BLOB_URI"]
        ds = VectorStoreDataSource(asset_identifier=asset_uri, asset_type=VectorStoreDataSourceAssetType.URI_ASSET)
        vector_store = await agents_client.vector_stores.create_and_poll(data_sources=[ds], name="sample_vector_store")
        print(f"Created vector store, vector store ID: {vector_store.id}")
        vector_store_files = {}
        async for fle in agents_client.vector_store_files.list(vector_store.id):
            uploaded_file = await agents_client.files.get(fle.id)
            vector_store_files[fle.id] = uploaded_file.filename

        # Create a file search tool
        file_search_tool = FileSearchTool(vector_store_ids=[vector_store.id])

        # Notices that FileSearchTool as tool and tool_resources must be added or the agent unable to search the file
        agent = await agents_client.create_agent(
            model=os.environ["MODEL_DEPLOYMENT_NAME"],
            name="my-agent",
            instructions="You are helpful agent",
            tools=file_search_tool.definitions,
            tool_resources=file_search_tool.resources,
        )
        print(f"Created agent, agent ID: {agent.id}")

        thread = await agents_client.threads.create()
        print(f"Created thread, thread ID: {thread.id}")

        message = await agents_client.messages.create(
            thread_id=thread.id, role="user", content="What feature does Smart Eyewear offer?"
        )
        print(f"Created message, message ID: {message.id}")

        run = await agents_client.runs.create_and_process(thread_id=thread.id, agent_id=agent.id)
        print(f"Created run, run ID: {run.id}")

        await agents_client.vector_stores.delete(vector_store.id)
        print("Deleted vector store")

        await agents_client.delete_agent(agent.id)
        print("Deleted agent")

        messages = agents_client.messages.list(thread_id=thread.id, order=ListSortOrder.ASCENDING)
        async for msg in messages:
            if msg.text_messages:
                last_text = msg.text_messages[-1].text.value
                for annotation in msg.text_messages[-1].text.annotations:

                    citation = vector_store_files.get(
                        annotation.file_citation.file_id, annotation.file_citation.file_id
                    )
                    last_text = last_text.replace(annotation.text, f" [{citation}]")
                print(f"{msg.role}: {last_text}")


if __name__ == "__main__":
    asyncio.run(main())



# coding=utf-8

from copy import deepcopy
from typing import Any, Awaitable
from typing_extensions import Self

from corehttp.rest import AsyncHttpResponse, HttpRequest
from corehttp.runtime import AsyncPipelineClient, policies

from .._utils.serialization import Deserializer, Serializer
from ._configuration import AdditionalPropertiesClientConfiguration
from .operations import (
    ExtendsDifferentSpreadFloatOperations,
    ExtendsDifferentSpreadModelArrayOperations,
    ExtendsDifferentSpreadModelOperations,
    ExtendsDifferentSpreadStringOperations,
    ExtendsFloatOperations,
    ExtendsModelArrayOperations,
    ExtendsModelOperations,
    ExtendsStringOperations,
    ExtendsUnknownDerivedOperations,
    ExtendsUnknownDiscriminatedOperations,
    ExtendsUnknownOperations,
    IsFloatOperations,
    IsModelArrayOperations,
    IsModelOperations,
    IsStringOperations,
    IsUnknownDerivedOperations,
    IsUnknownDiscriminatedOperations,
    IsUnknownOperations,
    MultipleSpreadOperations,
    SpreadDifferentFloatOperations,
    SpreadDifferentModelArrayOperations,
    SpreadDifferentModelOperations,
    SpreadDifferentStringOperations,
    SpreadFloatOperations,
    SpreadModelArrayOperations,
    SpreadModelOperations,
    SpreadRecordNonDiscriminatedUnion2Operations,
    SpreadRecordNonDiscriminatedUnion3Operations,
    SpreadRecordNonDiscriminatedUnionOperations,
    SpreadRecordUnionOperations,
    SpreadStringOperations,
)


class AdditionalPropertiesClient:  # pylint: disable=client-accepts-api-version-keyword,too-many-instance-attributes
    """Tests for additional properties of models.

    :ivar extends_unknown: ExtendsUnknownOperations operations
    :vartype extends_unknown:
     typetest.property.additionalproperties.aio.operations.ExtendsUnknownOperations
    :ivar extends_unknown_derived: ExtendsUnknownDerivedOperations operations
    :vartype extends_unknown_derived:
     typetest.property.additionalproperties.aio.operations.ExtendsUnknownDerivedOperations
    :ivar extends_unknown_discriminated: ExtendsUnknownDiscriminatedOperations operations
    :vartype extends_unknown_discriminated:
     typetest.property.additionalproperties.aio.operations.ExtendsUnknownDiscriminatedOperations
    :ivar is_unknown: IsUnknownOperations operations
    :vartype is_unknown: typetest.property.additionalproperties.aio.operations.IsUnknownOperations
    :ivar is_unknown_derived: IsUnknownDerivedOperations operations
    :vartype is_unknown_derived:
     typetest.property.additionalproperties.aio.operations.IsUnknownDerivedOperations
    :ivar is_unknown_discriminated: IsUnknownDiscriminatedOperations operations
    :vartype is_unknown_discriminated:
     typetest.property.additionalproperties.aio.operations.IsUnknownDiscriminatedOperations
    :ivar extends_string: ExtendsStringOperations operations
    :vartype extends_string:
     typetest.property.additionalproperties.aio.operations.ExtendsStringOperations
    :ivar is_string: IsStringOperations operations
    :vartype is_string: typetest.property.additionalproperties.aio.operations.IsStringOperations
    :ivar spread_string: SpreadStringOperations operations
    :vartype spread_string:
     typetest.property.additionalproperties.aio.operations.SpreadStringOperations
    :ivar extends_float: ExtendsFloatOperations operations
    :vartype extends_float:
     typetest.property.additionalproperties.aio.operations.ExtendsFloatOperations
    :ivar is_float: IsFloatOperations operations
    :vartype is_float: typetest.property.additionalproperties.aio.operations.IsFloatOperations
    :ivar spread_float: SpreadFloatOperations operations
    :vartype spread_float:
     typetest.property.additionalproperties.aio.operations.SpreadFloatOperations
    :ivar extends_model: ExtendsModelOperations operations
    :vartype extends_model:
     typetest.property.additionalproperties.aio.operations.ExtendsModelOperations
    :ivar is_model: IsModelOperations operations
    :vartype is_model: typetest.property.additionalproperties.aio.operations.IsModelOperations
    :ivar spread_model: SpreadModelOperations operations
    :vartype spread_model:
     typetest.property.additionalproperties.aio.operations.SpreadModelOperations
    :ivar extends_model_array: ExtendsModelArrayOperations operations
    :vartype extends_model_array:
     typetest.property.additionalproperties.aio.operations.ExtendsModelArrayOperations
    :ivar is_model_array: IsModelArrayOperations operations
    :vartype is_model_array:
     typetest.property.additionalproperties.aio.operations.IsModelArrayOperations
    :ivar spread_model_array: SpreadModelArrayOperations operations
    :vartype spread_model_array:
     typetest.property.additionalproperties.aio.operations.SpreadModelArrayOperations
    :ivar spread_different_string: SpreadDifferentStringOperations operations
    :vartype spread_different_string:
     typetest.property.additionalproperties.aio.operations.SpreadDifferentStringOperations
    :ivar spread_different_float: SpreadDifferentFloatOperations operations
    :vartype spread_different_float:
     typetest.property.additionalproperties.aio.operations.SpreadDifferentFloatOperations
    :ivar spread_different_model: SpreadDifferentModelOperations operations
    :vartype spread_different_model:
     typetest.property.additionalproperties.aio.operations.SpreadDifferentModelOperations
    :ivar spread_different_model_array: SpreadDifferentModelArrayOperations operations
    :vartype spread_different_model_array:
     typetest.property.additionalproperties.aio.operations.SpreadDifferentModelArrayOperations
    :ivar extends_different_spread_string: ExtendsDifferentSpreadStringOperations operations
    :vartype extends_different_spread_string:
     typetest.property.additionalproperties.aio.operations.ExtendsDifferentSpreadStringOperations
    :ivar extends_different_spread_float: ExtendsDifferentSpreadFloatOperations operations
    :vartype extends_different_spread_float:
     typetest.property.additionalproperties.aio.operations.ExtendsDifferentSpreadFloatOperations
    :ivar extends_different_spread_model: ExtendsDifferentSpreadModelOperations operations
    :vartype extends_different_spread_model:
     typetest.property.additionalproperties.aio.operations.ExtendsDifferentSpreadModelOperations
    :ivar extends_different_spread_model_array: ExtendsDifferentSpreadModelArrayOperations
     operations
    :vartype extends_different_spread_model_array:
     typetest.property.additionalproperties.aio.operations.ExtendsDifferentSpreadModelArrayOperations
    :ivar multiple_spread: MultipleSpreadOperations operations
    :vartype multiple_spread:
     typetest.property.additionalproperties.aio.operations.MultipleSpreadOperations
    :ivar spread_record_union: SpreadRecordUnionOperations operations
    :vartype spread_record_union:
     typetest.property.additionalproperties.aio.operations.SpreadRecordUnionOperations
    :ivar spread_record_non_discriminated_union: SpreadRecordNonDiscriminatedUnionOperations
     operations
    :vartype spread_record_non_discriminated_union:
     typetest.property.additionalproperties.aio.operations.SpreadRecordNonDiscriminatedUnionOperations
    :ivar spread_record_non_discriminated_union2: SpreadRecordNonDiscriminatedUnion2Operations
     operations
    :vartype spread_record_non_discriminated_union2:
     typetest.property.additionalproperties.aio.operations.SpreadRecordNonDiscriminatedUnion2Operations
    :ivar spread_record_non_discriminated_union3: SpreadRecordNonDiscriminatedUnion3Operations
     operations
    :vartype spread_record_non_discriminated_union3:
     typetest.property.additionalproperties.aio.operations.SpreadRecordNonDiscriminatedUnion3Operations
    :keyword endpoint: Service host. Default value is "http://localhost:3000".
    :paramtype endpoint: str
    """

    def __init__(  # pylint: disable=missing-client-constructor-parameter-credential
        self, *, endpoint: str = "http://localhost:3000", **kwargs: Any
    ) -> None:
        _endpoint = "{endpoint}"
        self._config = AdditionalPropertiesClientConfiguration(endpoint=endpoint, **kwargs)

        _policies = kwargs.pop("policies", None)
        if _policies is None:
            _policies = [
                self._config.headers_policy,
                self._config.user_agent_policy,
                self._config.proxy_policy,
                policies.ContentDecodePolicy(**kwargs),
                self._config.retry_policy,
                self._config.authentication_policy,
                self._config.logging_policy,
            ]
        self._client: AsyncPipelineClient = AsyncPipelineClient(endpoint=_endpoint, policies=_policies, **kwargs)

        self._serialize = Serializer()
        self._deserialize = Deserializer()
        self._serialize.client_side_validation = False
        self.extends_unknown = ExtendsUnknownOperations(self._client, self._config, self._serialize, self._deserialize)
        self.extends_unknown_derived = ExtendsUnknownDerivedOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.extends_unknown_discriminated = ExtendsUnknownDiscriminatedOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.is_unknown = IsUnknownOperations(self._client, self._config, self._serialize, self._deserialize)
        self.is_unknown_derived = IsUnknownDerivedOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.is_unknown_discriminated = IsUnknownDiscriminatedOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.extends_string = ExtendsStringOperations(self._client, self._config, self._serialize, self._deserialize)
        self.is_string = IsStringOperations(self._client, self._config, self._serialize, self._deserialize)
        self.spread_string = SpreadStringOperations(self._client, self._config, self._serialize, self._deserialize)
        self.extends_float = ExtendsFloatOperations(self._client, self._config, self._serialize, self._deserialize)
        self.is_float = IsFloatOperations(self._client, self._config, self._serialize, self._deserialize)
        self.spread_float = SpreadFloatOperations(self._client, self._config, self._serialize, self._deserialize)
        self.extends_model = ExtendsModelOperations(self._client, self._config, self._serialize, self._deserialize)
        self.is_model = IsModelOperations(self._client, self._config, self._serialize, self._deserialize)
        self.spread_model = SpreadModelOperations(self._client, self._config, self._serialize, self._deserialize)
        self.extends_model_array = ExtendsModelArrayOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.is_model_array = IsModelArrayOperations(self._client, self._config, self._serialize, self._deserialize)
        self.spread_model_array = SpreadModelArrayOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.spread_different_string = SpreadDifferentStringOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.spread_different_float = SpreadDifferentFloatOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.spread_different_model = SpreadDifferentModelOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.spread_different_model_array = SpreadDifferentModelArrayOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.extends_different_spread_string = ExtendsDifferentSpreadStringOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.extends_different_spread_float = ExtendsDifferentSpreadFloatOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.extends_different_spread_model = ExtendsDifferentSpreadModelOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.extends_different_spread_model_array = ExtendsDifferentSpreadModelArrayOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.multiple_spread = MultipleSpreadOperations(self._client, self._config, self._serialize, self._deserialize)
        self.spread_record_union = SpreadRecordUnionOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.spread_record_non_discriminated_union = SpreadRecordNonDiscriminatedUnionOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.spread_record_non_discriminated_union2 = SpreadRecordNonDiscriminatedUnion2Operations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.spread_record_non_discriminated_union3 = SpreadRecordNonDiscriminatedUnion3Operations(
            self._client, self._config, self._serialize, self._deserialize
        )

    def send_request(
        self, request: HttpRequest, *, stream: bool = False, **kwargs: Any
    ) -> Awaitable[AsyncHttpResponse]:
        """Runs the network request through the client's chained policies.

        >>> from corehttp.rest import HttpRequest
        >>> request = HttpRequest("GET", "https://www.example.org/")
        <HttpRequest [GET], url: 'https://www.example.org/'>
        >>> response = await client.send_request(request)
        <AsyncHttpResponse: 200 OK>

        For more information on this code flow, see https://aka.ms/azsdk/dpcodegen/python/send_request

        :param request: The network request you want to make. Required.
        :type request: ~corehttp.rest.HttpRequest
        :keyword bool stream: Whether the response payload will be streamed. Defaults to False.
        :return: The response of your network call. Does not do error handling on your response.
        :rtype: ~corehttp.rest.AsyncHttpResponse
        """

        request_copy = deepcopy(request)
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }

        request_copy.url = self._client.format_url(request_copy.url, **path_format_arguments)
        return self._client.send_request(request_copy, stream=stream, **kwargs)  # type: ignore

    async def close(self) -> None:
        await self._client.close()

    async def __aenter__(self) -> Self:
        await self._client.__aenter__()
        return self

    async def __aexit__(self, *exc_details: Any) -> None:
        await self._client.__aexit__(*exc_details)
